/*
 * libkarma/karma.h
 *
 * Copyright (c) Frank Zschockelt <libkarma@freakysoft.de> 2004
 *
 * You may distribute and modify this program under the terms of 
 * the GNU GPL, version 2 or later.
 *
 */

#ifndef KARMA_H
#define KARMA_H

#include <inttypes.h>

#define RIOPORT 8302
#define HEADERLENGTH 8
#define SALTLENGTH 16
#define DETAILSLENGTH 32

#define RIO_HEADER "\x52\x69\xC5\x8D" /* Ri~o */

#define GET_PROTOCOL_VERSION    0
#define NAK                     1
#define BUSY                    2
#define GET_AUTHENTICATION_SALT 3
#define AUTHENTICATE            4
#define GET_DEVICE_DETAILS      5
#define GET_STORAGE_DETAILS     6
#define GET_DEVICE_SETTINGS     7
#define UPDATE_DEVICE_SETTINGS  8
#define REQUEST_IO_LOCK         9
#define RELEASE_IO_LOCK        10
#define PREPARE                11
#define WRITE_FILE_CHUNK       12
#define GET_ALL_FILE_DETAILS   13
#define GET_FILE_DETAILS       14
#define UPDATE_FILE_DETAILS    15
#define READ_FILE_CHUNK        16
#define DELETE_FILE            17
#define FORMAT_STORAGE         18
#define HANGUP                 19
#define DEVICE_OPERATION       20

#define IO_LOCK_R 0 
#define IO_LOCK_W 1


/*Help-Functions:*/
int connectkarma(char * ip);
int send_request(int rio, uint32_t identifier, char * payload, int plen);
unsigned char * generate_rid(int file, int offset, int length);

uint32_t htorl(uint32_t hostlong);
uint32_t rtohl(uint32_t hostlong);
uint64_t htorll(uint64_t hostlong);
uint64_t rtohll(uint64_t hostlong);

/*Basic Protocol Functions*/
int      get_protocol_version   (int rio, uint32_t * major_version,
                                 uint32_t * minor_version);
int32_t  get_authentication_salt(int rio, char ** salt);
uint32_t authenticate           (int rio, char * pass);
int      get_device_details     (int rio, char **name, 
				 char **version, uint32_t * storagedevices);
int      get_storage_details    (int rio, uint32_t storage_id, 
                                 uint32_t * n_files, uint64_t * s_size, 
				 uint64_t * f_space, 
				 uint32_t * highest_file_id);
int      get_device_settings    (int rio, char ** properties);
int32_t  update_device_settings (int rio, char * properties);
int32_t  request_io_lock        (int rio, uint32_t type);
int32_t  release_io_lock        (int rio);
int32_t  prepare                (int rio, uint64_t size, 
				 uint32_t * file_id, uint32_t storage_id);
int32_t  write_file_chunk       (int rio, uint64_t offset, uint64_t size, 
                                 uint32_t file_id, uint32_t storage_id, 
				 char * data);
int32_t  get_all_file_details   (int rio, char ** properties);
int32_t  get_file_details       (int rio, uint32_t file_id, 
				 char ** properties);
int32_t  update_file_details    (int rio, uint32_t file_id, 
				 char * properties);
int32_t  read_file_chunk        (int rio, uint64_t offset, uint64_t size,
                                 uint32_t file_id, char ** data, 
				 uint64_t * retsize);
int32_t  delete_file            (int rio, uint32_t file_id);
int32_t  format_storage         (int rio, uint32_t storage_id);
int32_t  hangup                 (int rio);
int32_t  device_operation       (int rio, uint64_t size, char * data,
				 char ** newdata);

/*Advanced Protocol Functions:*/
void update_database(int rio);

#endif /* KARMA_H */
