/*
 * libkarma/chprop.c
 *
 * Copyright (c) Frank Zschockelt <libkarma@freakysoft.de> 2004
 *
 * You may distribute and modify this program under the terms of 
 * the GNU GPL, version 2 or later.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>

#include "karma.h"
#include "properties.h"

#define CHECK(x) x?"failed":"ok"

void usage(void);

void usage(void)
{
	printf("usage: chprop -a hostname/ip [-p password] fid key value\n");
}

int main(int argc, char * argv[])
{
	int c;
	int karma;
	char *host=NULL;
	char *p=NULL;
	uint32_t fid;

	while ((c = getopt(argc, argv, "ha:p:s:")) != -1) {
		switch(c) {
			case 'h':
				usage();
				return -1;
				break;
			case 'a':
				host=strdup(optarg);
				break;
			case 'p':
				p=strdup(optarg);
				break;
			case '?':
				printf("unknown arg %c\n", optopt);
				break;
		}
	}
	
	if((!host) ||(optind+3 != argc)){
		usage();
		if(p) free(p);
		return -1;
	}
	
	karma=connectkarma(host);
	free(host);
	
	if(karma < 0){
		usage();
		if(p) free(p);
		return -1;
	}

	init_properties("ISO8859-15");
	
	if(p){
		printf("authenticate: %i\n", authenticate(karma, p));
		free(p);
	}else printf("authenticate: %i\n", authenticate(karma, ""));
	printf("request_io_lock: %s\n", CHECK(request_io_lock(karma,IO_LOCK_R)));
	
	get_device_settings(karma, &p);
	import_properties(p);
	free(p);
	
	fid=strtoul(argv[optind], (char **) NULL, 10);
	
	printf("get_file_details: %s\n", CHECK(get_file_details(karma, fid,&p)));
	import_properties(p);
	free(p);
  
	printf("request_io_lock: %s\n", CHECK(request_io_lock(karma,IO_LOCK_W)));
	change_property(fid, argv[optind+1], argv[optind+2]);
	p=export_property(fid);
	printf("update_file_details: %s\n", CHECK(update_file_details(karma, fid, p)));
	free(p);
	printf("release_io_lock: %s\n", CHECK(release_io_lock(karma))); 
	printf("hangup: %s\n", CHECK(hangup(karma)));
	close(karma);
	destroy_properties();
	return 0;
}
